import Vue from 'vue';

const classRegExp = (className) => new RegExp(`(^|\\s+)${className.toString().trim()}(\\s+|$)`, 'g');
const hasClass = (el, className) => classRegExp(className).test(el.className);
const addClass = (el, className) => {
    const classNames = className.split(' ');
    classNames.length > 1
        ? classNames.forEach(className => addClass(el, className))
        : hasClass(el, className) ||
            (el.className = `${el.className} ${className}`.trim());
};
const removeClass = (el, className) => {
    el.className = el.className.replace(classRegExp(className), ' ').trim();
};

//

const isPromise = promise =>
  Object.prototype.toString.call(promise) === '[object Promise]' ||
  promise instanceof Promise;

const DEFAULT_OPTIONS = {
  show: true,
  backdrop: true,
  destroy: false,
};

const NON_TRANSITION_ERR =
  'this modal item is not a Vue component, you should use `transition` component and emit `after-leave` event';

var script = {
  data() {
    return {
      modals: [],
      currModal: null,
    }
  },
  computed: {
    currModalId() {
      return this.currModal && this.currModal.id
    },
  },
  watch: {
    currModal: modal =>
      (modal ? addClass : removeClass)(document.body, 'modal-open'),
  },
  beforeCreate() {
    Object.defineProperty(Vue.prototype, '$modal', {
      value: this,
      writable: process.env.NODE_ENV === 'development',
    });
  },
  methods: {
    close(modalId, destroy) {
      modalId = modalId || this.currModalId;

      let modal;

      if (!modalId || !(modal = this.getModal(modalId)))
        return Promise.resolve()

      const { options } = modal;

      options.show = false;

      const modalItem = this.getModalItem(modalId);

      if (!modalItem) return Promise.reject(new TypeError(NON_TRANSITION_ERR))

      const callback = resolve => {
        options.destroy || destroy
          ? this.removeModal(modalId)
          : this.resetCurrModal(modalId);
        resolve();
      };

      return new Promise(resolve =>
        getComputedStyle(modalItem.$el).display === 'none'
          ? callback(resolve)
          : modalItem.$once('after-leave', () => callback(resolve)),
      )
    },
    closeAll(destroy = true, immediate = false) {
      let promise = Promise.resolve();

      destroy && immediate
        ? (this.modals = [])
        : this.modals.forEach(modal => {
            promise = promise.then(() => this.close(modal.id, destroy));
          });

      return promise
    },
    open(modal) {
      modal.id = modal.id || Date.now();
      return isPromise(modal.component)
        ? modal.component.then(component =>
            this.resolve(
              Object.assign(modal, {
                component: component.default || component,
              }),
            ),
          )
        : this.resolve(modal)
    },
    resolve(modal) {
      const { id, component, props, options } = modal;

      const m = this.getModal(id);

      if (m) {
        component && (m.component = component);
        modal = m;
      } else if (!component) {
        return Promise.reject(
          new ReferenceError('no component passed on initialization'),
        )
      }

      modal.props = { ...props };

      const opts = { ...DEFAULT_OPTIONS, ...options };

      if (!opts.show) {
        modal.options = opts;
        return Promise.resolve()
      }

      const promise = this.currModalId === id ? Promise.resolve() : this.close();

      return promise.then(() => {
        modal.options = opts;
        m || this.modals.push(modal);
        this.currModal = modal;

        return new Promise((resolve, reject) =>
          this.$nextTick(() => {
            const modalItem = this.getModalItem(id);
            modalItem
              ? modalItem.$once('after-enter', () => resolve(modal))
              : reject(new TypeError(NON_TRANSITION_ERR));
          }),
        )
      })
    },
    getModal(modalId) {
      return this.modals.find(m => m.id === modalId)
    },
    getModalIndex(modalId) {
      return this.modals.findIndex(m => m.id === modalId)
    },
    getModalRef(modalId) {
      return this.$refs.modal[this.getModalIndex(modalId)]
    },
    getModalItem(modalId) {
      const modalRef = this.getModalRef(modalId);
      return modalRef && modalRef.$children[0]
    },
    resetCurrModal(modalId) {
      if (modalId === this.currModalId) {
        this.currModal = null;
      }
    },
    removeModal(modalId) {
      const modalIndex = this.getModalIndex(modalId);
      modalIndex + 1 && this.modals.splice(modalIndex, 1);
      this.resetCurrModal(modalId);
    },
    handleBackdrop(e, id, backdrop) {
      if (e.target !== e.currentTarget || backdrop === 'static') {
        return
      }
      this.close(id);
    },
  },
};

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    const options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    let hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            const originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            const existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}

const isOldIE = typeof navigator !== 'undefined' &&
    /msie [6-9]\\b/.test(navigator.userAgent.toLowerCase());
function createInjector(context) {
    return (id, style) => addStyle(id, style);
}
let HEAD;
const styles = {};
function addStyle(id, css) {
    const group = isOldIE ? css.media || 'default' : id;
    const style = styles[group] || (styles[group] = { ids: new Set(), styles: [] });
    if (!style.ids.has(id)) {
        style.ids.add(id);
        let code = css.source;
        if (css.map) {
            // https://developer.chrome.com/devtools/docs/javascript-debugging
            // this makes source maps inside style tags work properly in Chrome
            code += '\n/*# sourceURL=' + css.map.sources[0] + ' */';
            // http://stackoverflow.com/a/26603875
            code +=
                '\n/*# sourceMappingURL=data:application/json;base64,' +
                    btoa(unescape(encodeURIComponent(JSON.stringify(css.map)))) +
                    ' */';
        }
        if (!style.element) {
            style.element = document.createElement('style');
            style.element.type = 'text/css';
            if (css.media)
                style.element.setAttribute('media', css.media);
            if (HEAD === undefined) {
                HEAD = document.head || document.getElementsByTagName('head')[0];
            }
            HEAD.appendChild(style.element);
        }
        if ('styleSheet' in style.element) {
            style.styles.push(code);
            style.element.styleSheet.cssText = style.styles
                .filter(Boolean)
                .join('\n');
        }
        else {
            const index = style.ids.size - 1;
            const textNode = document.createTextNode(code);
            const nodes = style.element.childNodes;
            if (nodes[index])
                style.element.removeChild(nodes[index]);
            if (nodes.length)
                style.element.insertBefore(textNode, nodes[index]);
            else
                style.element.appendChild(textNode);
        }
    }
}

/* script */
const __vue_script__ = script;

/* template */
var __vue_render__ = function() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _vm.modals.length
    ? _c(
        "div",
        [
          _vm.currModal && _vm.currModal.options.backdrop
            ? _c("div", { staticClass: "modal-backdrop" })
            : _vm._e(),
          _vm._v(" "),
          _vm._l(_vm.modals, function(ref) {
            var id = ref.id;
            var component = ref.component;
            var props = ref.props;
            var options = ref.options;
            return _c(
              component,
              _vm._b(
                {
                  directives: [
                    {
                      name: "show",
                      rawName: "v-show",
                      value: options.show,
                      expression: "options.show"
                    }
                  ],
                  key: id,
                  ref: "modal",
                  refInFor: true,
                  tag: "component",
                  nativeOn: {
                    click: function($event) {
                      return (function(e) {
                        return _vm.handleBackdrop(e, id, options.backdrop)
                      })($event)
                    }
                  }
                },
                "component",
                props,
                false
              )
            )
          })
        ],
        2
      )
    : _vm._e()
};
var __vue_staticRenderFns__ = [];
__vue_render__._withStripped = true;

  /* style */
  const __vue_inject_styles__ = function (inject) {
    if (!inject) return
    inject("data-v-18948bea_0", { source: ".modal-open {\n  overflow: hidden;\n}\n.modal-backdrop {\n  position: fixed;\n  top: 0;\n  right: 0;\n  bottom: 0;\n  left: 0;\n  z-index: 1040;\n  background-color: #000;\n  opacity: 0.5;\n  filter: alpha(opacity=50);\n}", map: undefined, media: undefined });

  };
  /* scoped */
  const __vue_scope_id__ = undefined;
  /* module identifier */
  const __vue_module_identifier__ = undefined;
  /* functional template */
  const __vue_is_functional_template__ = false;
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__ = normalizeComponent(
    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    false,
    createInjector,
    undefined,
    undefined
  );

//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//
//

var script$1 = {
  props: {
    id: [Number, String],
    header: String,
    footer: Boolean,
    disabled: Boolean,
    confirmText: String,
    cancelText: String,
    transition: {
      type: String,
      default: 'fade',
    },
  },
  methods: {
    closeModal() {
      this.$modal.close();
    },
    confirmModal() {
      if (this.disabled) {
        return
      }
      this.$emit('confirm');
    },
  },
};

/* script */
const __vue_script__$1 = script$1;

/* template */
var __vue_render__$1 = function() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c(
    "transition",
    {
      attrs: { name: _vm.transition },
      on: {
        "after-enter": function($event) {
          return _vm.$emit("after-enter")
        },
        "after-leave": function($event) {
          return _vm.$emit("after-leave")
        }
      }
    },
    [
      _c(
        "div",
        {
          staticClass: "modal",
          attrs: { id: _vm.id, tabindex: "-1", role: "dialog" }
        },
        [
          _c("div", { staticClass: "modal-dialog" }, [
            _c(
              "div",
              { staticClass: "modal-content" },
              [
                _vm.$slots.header
                  ? _c(
                      "div",
                      { staticClass: "modal-header" },
                      [_vm._t("header")],
                      2
                    )
                  : _vm.header
                  ? _c("div", { staticClass: "modal-header" }, [
                      _c(
                        "button",
                        {
                          staticClass: "close",
                          attrs: { type: "button" },
                          on: { click: _vm.closeModal }
                        },
                        [_vm._v("×")]
                      ),
                      _vm._v(" "),
                      _vm.header
                        ? _c("h4", {
                            staticClass: "modal-title",
                            domProps: { innerHTML: _vm._s(_vm.header) }
                          })
                        : _vm._e()
                    ])
                  : _vm._e(),
                _vm._v(" "),
                _vm.$slots.body
                  ? _vm._t("body")
                  : _c(
                      "div",
                      { staticClass: "modal-body" },
                      [_vm._t("default")],
                      2
                    ),
                _vm._v(" "),
                _vm.$slots.footer
                  ? _c(
                      "div",
                      { staticClass: "modal-footer" },
                      [_vm._t("footer")],
                      2
                    )
                  : _vm.footer
                  ? _c("div", { staticClass: "modal-footer" }, [
                      _c(
                        "div",
                        {
                          staticClass: "btn btn-default",
                          on: { click: _vm.closeModal }
                        },
                        [
                          _vm._v(
                            "\n            " +
                              _vm._s(_vm.cancelText || "取消") +
                              "\n          "
                          )
                        ]
                      ),
                      _vm._v(" "),
                      _c(
                        "div",
                        {
                          staticClass: "btn btn-primary",
                          attrs: { disabled: _vm.disabled },
                          on: { click: _vm.confirmModal }
                        },
                        [
                          _vm._v(
                            "\n            " +
                              _vm._s(_vm.confirmText || "确定") +
                              "\n          "
                          )
                        ]
                      )
                    ])
                  : _vm._e()
              ],
              2
            )
          ])
        ]
      )
    ]
  )
};
var __vue_staticRenderFns__$1 = [];
__vue_render__$1._withStripped = true;

  /* style */
  const __vue_inject_styles__$1 = function (inject) {
    if (!inject) return
    inject("data-v-9a70c88e_0", { source: ".modal {\n  overflow: hidden;\n  position: fixed;\n  top: 0;\n  right: 0;\n  bottom: 0;\n  left: 0;\n  z-index: 1050;\n  -webkit-overflow-scrolling: touch;\n  outline: 0;\n}\n.modal-open .modal {\n  overflow-x: hidden;\n  overflow-y: auto;\n}\n.modal-dialog {\n  position: relative;\n  width: auto;\n  margin: 10px;\n}\n.modal-content {\n  position: relative;\n  border: 1px solid #999;\n  border: 1px solid rgba(0, 0, 0, 0.2);\n  border-radius: 6px;\n  box-shadow: 0 3px 9px rgba(0, 0, 0, 0.5);\n  background-color: #fff;\n  background-clip: padding-box;\n  outline: 0;\n}\n.modal-header {\n  padding: 15px;\n  border-bottom: 1px solid #e5e5e5;\n}\n.modal-header:before, .modal-header:after {\n  content: \" \";\n  display: table;\n}\n.modal-header:after {\n  clear: both;\n}\n.modal-header .close {\n  margin-top: -2px;\n}\n.modal-title {\n  margin: 0;\n  line-height: 1.4285714286;\n}\n.modal-body {\n  position: relative;\n  padding: 15px;\n}\n.modal-footer {\n  padding: 15px;\n  text-align: right;\n  border-top: 1px solid #e5e5e5;\n}\n.modal-footer:before, .modal-footer:after {\n  content: \" \";\n  display: table;\n}\n.modal-footer:after {\n  clear: both;\n}\n.modal-footer .btn + .btn {\n  margin-left: 5px;\n  margin-bottom: 0;\n}\n.modal-footer .btn-group .btn + .btn {\n  margin-left: -1px;\n}\n.modal-footer .btn-block + .btn-block {\n  margin-left: 0;\n}\n.modal-scrollbar-measure {\n  position: absolute;\n  top: -9999px;\n  width: 50px;\n  height: 50px;\n  overflow: scroll;\n}\n@media (min-width: 768px) {\n.modal-dialog {\n    width: 600px;\n    margin: 30px auto;\n}\n.modal-content {\n    box-shadow: 0 5px 15px rgba(0, 0, 0, 0.5);\n}\n.modal-sm {\n    width: 300px;\n}\n}\n@media (min-width: 992px) {\n.modal-lg {\n    width: 900px;\n}\n}", map: undefined, media: undefined });

  };
  /* scoped */
  const __vue_scope_id__$1 = undefined;
  /* module identifier */
  const __vue_module_identifier__$1 = undefined;
  /* functional template */
  const __vue_is_functional_template__$1 = false;
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__$1 = normalizeComponent(
    { render: __vue_render__$1, staticRenderFns: __vue_staticRenderFns__$1 },
    __vue_inject_styles__$1,
    __vue_script__$1,
    __vue_scope_id__$1,
    __vue_is_functional_template__$1,
    __vue_module_identifier__$1,
    false,
    createInjector,
    undefined,
    undefined
  );

export { __vue_component__ as Modal, __vue_component__$1 as ModalItem };
